#ifndef LDRAWABLE_H_
#define LDRAWABLE_H_

#include "LColor.h"

// This class contains the declaration of graphics primitive which are
// useful for widgets like canvases and images stored as memory objects.
// The 'drawable' term is borrowed from X.

// _Every thing_ is implemented in sysdep/LDrawable.cc.

#include "sysdep/LDrawable.h"

class LDrawable{
protected:
  // This macro expands to the declaration of system-dependant fields:
  LDRAWABLE_SYSDEP_FIELDS

  // Graphic context fields:
  LColor CurrentColor;
  //LColor CurrentFillColor; // Should I do this?
  int w,h;
public:
  // Constructors and destructor:
  LDrawable();                     // default constructor
  // Copy constructors should be implemented in sub-classes.
  ~LDrawable();                    // destructor

  // Access methods implemented in sysdep/LDrawable.cc:
  virtual void SetColor(const LColor c);
  virtual const LColor GetColor() const;

  // Public methods implementing graphic primitives; their meaning is
  // obvious. They are defined in sysdep/LDrawable.cc, but these methods
  // could also be dummy: depending on the implementation they could
  // be overridden in every inherited class.
  virtual void Clear();
  virtual void DrawPixel(int x,int y);
  virtual void DrawPixel(int x,int y,const LColor c);
  virtual void DrawLine(int x1,int y1,int x2,int y2);
  virtual void DrawLine(int x1,int y1,int x2,int y2,const LColor c);
  virtual void DrawFullRectangle(int x1,int y1,int x2,int y2);
  virtual void DrawFullRectangle(int x1,int y1,int x2,int y2,const LColor c);
  virtual void DrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3);
  virtual void DrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3,const LColor c);

  virtual void CopyAreaFrom(int x1dest,int y1dest,
                            LDrawable &source,int x1,int y1,int x2,int y1);
};

#endif //#ifndef LDRAWABLE_H_

