//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "../../LButton.h"
#include "../../LAttributes.h"
#include "LSysDep.h"

extern "C"{
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
}

static void UniversalLButtonCallBack(Widget w,XtPointer callData,
                                     XtPointer clientData){
  // This dummy callback simply calls the real LCallBack function, which
  // can be found thru callData, which in its turn contains a pointer to the
  // LTakeshi widget:
  LCallBack c=((LButton*)callData)->GetCallBack(LAEffect);

  // I know c is not null, because if LCallBack is null the Xt Callback list
  // is cleared; so, if we are at this point, a callback does exist.
  c(*(LButton*)callData,LVoidEvent);
}

void LButton::Apply(){
  if(TestAndSet(LAW))
    XtVaSetValues(SysDep.widget,XtNwidth,GetInt(LAW),
                                0); // Apply width changes
  if(TestAndSet(LAH))
    XtVaSetValues(SysDep.widget,XtNheight,GetInt(LAH),
                                0); // Apply height changes
  if(TestAndSet(LAX))
    XtVaSetValues(SysDep.widget,XtNhorizDistance,GetInt(LAX),
                                0); // Apply x position changes
  if(TestAndSet(LAY))
    XtVaSetValues(SysDep.widget,XtNvertDistance,GetInt(LAY),
                                0); // Apply y position changes
  if(TestAndSet(LAEffect)){         // Apply effect changes: start
    // I want to call _one_ callback: remove the current ones...
    XtRemoveAllCallbacks(SysDep.widget,XtNcallback);

    // ...and add the new one, if it's not null
    if(GetCallBack(LAEffect)!=0)
      XtAddCallback(SysDep.widget,XtNcallback,UniversalLButtonCallBack,this);
    // The universal (Xt) callback simply calls the real (LTakeshi) LCallBack
  }                                 // Apply effect changes: end

  if(TestAndSet(LAImage)){          // Apply image changes: start
    unsigned int w,h;
    int xhot,yhot;

    // Free the storage of p if this is not the first time is used
    if(SysDep.PrivatePixmap!=0)
      XFreePixmap(GlobalSysDep.display,SysDep.PrivatePixmap);

    // Now create the pixmap...
    if(XReadBitmapFile(GlobalSysDep.display,GlobalSysDep.drawable,
       GetString(LAImage).ToCharStar(),&w,&h,
       &(SysDep.PrivatePixmap),&xhot,&yhot)==BitmapSuccess)
      XtVaSetValues(SysDep.widget,               // ...and show it if it was
                    XtNbitmap, SysDep.PrivatePixmap,// created with success.
                    0);
    else
      XtVaSetValues(SysDep.widget,XtNlabel,  "[BITMAP]",
                                  XtNwidth,  GetInt(LAW),
                                  0); // Apply text changes
  }                                   // Apply image changes: end
  if(TestAndSet(LAText))
    XtVaSetValues(SysDep.widget,XtNlabel,  GetString(LAText).ToCharStar(),
                                XtNwidth,  GetInt(LAW),
                                0); // Apply text changes
}

void LButton::SystemDependantInit(){
  SysDep.PrivatePixmap=0; // I want to remember if I used PrivatePixmap
  SysDep.widget=XtVaCreateManagedWidget("LButton",
                commandWidgetClass,parent->SysDep.area,
                XtNbottom,        XtChainTop,
                XtNtop,           XtChainTop,
                XtNleft,          XtChainLeft,
                XtNright,         XtChainLeft,
                XtNjustify,       XtJustifyCenter,
                XtNhorizDistance, 1,
                XtNvertDistance,  1,
                XtNresizable,     TRUE,// To do: this might cause problems
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                0);
  Apply();

  // Now set the translations table:

  // First define the translations in a readable way, to be parsed by
  // XtParseTranslationTable():
  char ReadableTable[]="<KeyPress>Return: set() notify() unset()";// Activate
                                   // LTakeshi buttons also by pressing Enter
                       
  // Augment translations for the widget:
  XtAugmentTranslations(SysDep.widget,                           // widget
                         XtParseTranslationTable(ReadableTable));// table
}

void LButton::SystemDependantDone(){
  // Free PrivatePixmap if it was used
  if(SysDep.PrivatePixmap!=0)
    XFreePixmap(GlobalSysDep.display,SysDep.PrivatePixmap);
}










