//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


extern "C"{
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Form.h>
}

#include "LSysDep.h"
#include "../../LAttributes.h"
#include "../../LDrawable.h"
#include "../../LCanvas.h"
#include "../../LPixmap.h"

// An hash table is needed:
#include "../../../Lhash.h"


// Global objects used in this module:
XtTranslations ParsedTranslations=0;// The translations for LCanvas objects,
                                    // set once for all in
                                    // LCanvas::SystemDependantInit().

// Methods' definitions start here.

void LCanvas::Apply(){
  if(TestAndSet(LAW)){
    int w; // Temporary variable used to avoid calling LCanvas::GetInt() twice

    // Apply width changes:
    XtVaSetValues(SysDep.area,XtNwidth,w=GetInt(LAW),
                                0); // Apply width changes
    // Move SysDep.stupid to avoid Athena messes up width:
    XtVaSetValues(SysDepAsCanvas.stupid,
                  XtNhorizDistance, w-SysDepAsCanvas.distance-1,
                  0);

    // Update CanvasPixmap:
    CanvasPixmap.Resize(GetInt(LAW),GetInt(LAH));

    // Make the change visible to LDrawable methods:
    LDrawable::w=w;
  }
  if(TestAndSet(LAH)){
    int h; // Temporary variable used to avoid calling LCanvas::GetInt() twice

    // Apply height changes:
    XtVaSetValues(SysDep.area,XtNheight,h=GetInt(LAH),
                                0); // Apply height changes
    // Move SysDep.stupid to avoid Athena messes up height:
    XtVaSetValues(SysDepAsCanvas.stupid,
                  XtNvertDistance, h-SysDepAsCanvas.distance-1,
                  0);

    // Update CanvasPixmap:
    CanvasPixmap.Resize(GetInt(LAW),GetInt(LAH));
                   
    // Make the change visible to LDrawable methods:
    LDrawable::h=h;
  }
  if(TestAndSet(LAX))
    XtVaSetValues(SysDep.area,XtNhorizDistance,GetInt(LAX),
                                0); // Apply x position changes
  if(TestAndSet(LAY))
    XtVaSetValues(SysDep.area,XtNvertDistance,GetInt(LAY),
                                0); // Apply y position changes
}

// Action procedure for the Redraw X event:
void LCanvasRedrawCanvas(Widget w,XEvent *e,String *s,Cardinal *c){
  // We know the Xt widget associated with the event, but not the LTakeshi
  // widget: let's obtain it from the WidgetLWidgetStar. I know there's no
  // need to control that the pointer is not null: it has been me to fill
  // the hash table (and I touch it only in this module), and I'm very
  // careful with these things 8-).
  LCanvas *LTakeshiWidget=(LCanvas*)(WidgetToLWidgetStar[w]);

  // Re-draw the SysDepAsCanvas.Image pixmap:
  LTakeshiWidget->FastCopyAreaFrom(0,0,                       // dest x / y
                               LTakeshiWidget->CanvasPixmap,  // source
                               0,0,                           // source x1/y1
                               LTakeshiWidget->GetInt(LAW),   // source x2
                               LTakeshiWidget->GetInt(LAH));  // source y2
}

// Action procedure calling the actual LCallBack function:
void LCanvasMouseClick(Widget w,XEvent *e,String *s,Cardinal *c){
  cerr<<"MouseClick (";
  cerr<<e->xbutton.x<<","<<e->xbutton.y<<")\n";

  // Prepare the LEvent:
  static LEvent Event;
  Event.MouseEvent.x=e->xbutton.x;
  Event.MouseEvent.y=e->xbutton.y;

  // Invoke the callback:
  CallLCallBack(w,LAOnMouseClick,Event);
}

// Action procedure calling the actual LCallBack function:
void LCanvasMouseDoubleClick(Widget w,XEvent *e,String *s,Cardinal *c){
  cerr<<"MouseDoubleClick (";
  cerr<<e->xbutton.x<<","<<e->xbutton.y<<")\n";

  // Prepare the LEvent:
  static LEvent Event;
  Event.MouseEvent.x=e->xbutton.x;
  Event.MouseEvent.y=e->xbutton.y;

  // Invoke the callback:
  CallLCallBack(w,LAOnMouseDoubleClick,Event);
}

// Action procedure calling the actual LCallBack function:
void LCanvasMouseLeaveWindow(Widget w,XEvent *e,String *s,Cardinal *c){
  cerr<<"MouseLeaveWindow\n";

  // Invoke the callback:
  CallLCallBack(w,LAOnUnfocus,LVoidEvent);
}

// Action procedure calling the actual LCallBack function:
void LCanvasMouseEnterWindow(Widget w,XEvent *e,String *s,Cardinal *c){
  cerr<<"MouseEnterWindow\n";

  // Prepare the LEvent:
  static LEvent Event;
  Event.MouseEvent.x=e->xbutton.x;
  Event.MouseEvent.y=e->xbutton.y;

  // Invoke the callback:
  CallLCallBack(w,LAOnFocus,Event);
}

// Action procedure calling the actual LCallBack function:
void LCanvasKey(Widget w,XEvent *e,String *s,Cardinal *c){
  cerr<<"Key(";
  cerr<<e->xbutton.state<<","<<e->xbutton.button<<")\n";
  
  // Prepare the LEvent:
  static LEvent Event;
  Event.KeyboardEvent.key=keycode2LKey(e->xkey.keycode);

  // Invoke the callback:
  CallLCallBack(w,LAOnKey,Event);
}

// Action procedure calling the actual LCallBack function:
void LCanvasMouseMove(Widget w,XEvent *e,String *s,Cardinal *c){
  cerr<<"MouseMove (";
  cerr<<e->xbutton.x<<","<<e->xbutton.y<<")\n";

  // Prepare the LEvent:
  static LEvent Event;
  Event.MouseEvent.x=e->xbutton.x;
  Event.MouseEvent.y=e->xbutton.y;

  // Invoke the callback:
  CallLCallBack(w,LAOnMouseMove,Event);
}

void LCanvas::SystemDependantInit(){
  // Create the main Xt/Athena widget
  SysDep.area=XtVaCreateManagedWidget("LCanvas",
                  formWidgetClass,parent->SysDep.area,
                  XtNbottom,        XtChainTop,
                  XtNtop,           XtChainTop,
                  XtNleft,          XtChainLeft,
                  XtNright,         XtChainLeft,
                  XtNjustify,       XtJustifyLeft,
                  XtNhorizDistance, 1,
                  XtNvertDistance,  1,
                  XtNresizable,     TRUE,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                  0);

  // No need to create the graphic context: it's the LDrawable constructor to
  // do this.

  // Initialize drawable (inherited from LDrawable):
  LDrawable::drawable=XtWindow(SysDep.area);

  // Add an entry for the newly-created Xt widget in WidgetLWidgetStar:
  WidgetToLWidgetStar.insert(SysDep.area,  // key,  i.e. the Widget
                             this);        // data, i.e. the LWidget*

  // Create the stupid Xt/Athena widget: see comments in sysdep/LCanvas.h
  SysDepAsCanvas.stupid=XtVaCreateManagedWidget("LStupid",
                labelWidgetClass,    SysDep.area,
                XtNlabel,            "",
                XtNborderWidth,      0,
                XtNheight,           h=1,
                XtNwidth,            w=1,
                XtNbottom,           XtChainTop,
                XtNtop,              XtChainTop,
                XtNleft,             XtChainLeft,
                XtNright,            XtChainLeft,
                0);

  // Find defaultDistance, to be used to place SysDepAsCanvas.stupid:
  XtVaGetValues(SysDep.area,
                XtNdefaultDistance,    &(SysDepAsCanvas.distance),
                0);

  // Update CanvasPixmap:
  CanvasPixmap.Resize(GetInt(LAW),GetInt(LAH));
                 
  // Call LCanvas::Apply() to set attributes for the first time:
  Apply();

  // Create the actions and the translation table. There's no need to exec
  // this initialization more than once:
  if(ParsedTranslations==0){  // if the table hasn't been parsed yet
    // First create the action list:
    XtActionsRec TheActions[]={
      {"LCanvasRedrawCanvas",LCanvasRedrawCanvas},
      {"LCanvasMouseClick",       LCanvasMouseClick},
      {"LCanvasMouseDoubleClick", LCanvasMouseDoubleClick},
      {"LCanvasMouseLeaveWindow", LCanvasMouseLeaveWindow},
      {"LCanvasMouseEnterWindow", LCanvasMouseEnterWindow},
      {"LCanvasMouseMove",   LCanvasMouseMove},
      {"LCanvasKey",         LCanvasKey}
    };

    // Now add the actions:
    XtAppAddActions(GlobalSysDep.appContext, // application context
                    TheActions,              // action list
                    XtNumber(TheActions));   // number of actions

    // Define the translation table for LCanvas:
    char *HumanReadableTable= // To be parsed with XtParseTranslationTable()
         "<Expose>            : LCanvasRedrawCanvas()\n" // The canvas area must be
                                                  // redrawn at every X Expose
                                                  // event.
//         "<BtnDown>,<PtrMoved>,<BtnUp> : LCanvasDoubleClick()\n"
         "<BtnUp>             : LCanvasMouseClick()      \n"
         "<PtrMoved>          : LCanvasMouseMove()       \n"
         "<EnterWindow>       : LCanvasMouseEnterWindow()\n"
         "<LeaveWindow>       : LCanvasMouseLeaveWindow()\n"
         "<KeyPress>          : LCanvasKey()             \n"
         "<KeyRelease>        : LCanvasKey()        ";
    // Parse it:
    ParsedTranslations=XtParseTranslationTable(HumanReadableTable);

    // Ok. Now the table has been parsed, once for all. ParsedTranslations
    // will be used whenever a LCanvas is created.
  } // if(...

  // Add the translations:
  XtAugmentTranslations(SysDep.area,         // Xt widget
                        ParsedTranslations); // translations
}

void LCanvas::SystemDependantDone(){
  // Delete the entry for the Xt widget from the WidgetLWidgetStar:
  WidgetToLWidgetStar.remove(SysDep.area); // key, i.e. the Widget

  // No need to free the graphic context: it's the LDrawable destructor to
  // do this.
}

void LCanvas::FastClear(){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::Clear();
}

void LCanvas::Clear(){
  // Call the fast version:
  FastClear();

  // Update CanvasPixmap:
  CanvasPixmap.Clear();
}

void LCanvas::FastDrawPixel(int x,int y){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::DrawPixel(x,y);
}

void LCanvas::DrawPixel(int x,int y){
  // Call the fast version:
  FastDrawPixel(x,y);
  
  // Update CanvasPixmap:
  CanvasPixmap.DrawPixel(x,y);
}

void LCanvas::FastDrawPixel(int x,int y,const LColor c){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::DrawPixel(x,y,c);
}

void LCanvas::DrawPixel(int x,int y,const LColor c){
  // Call the fast version:
  FastDrawPixel(x,y,c);
  
  // Update CanvasPixmap:
  CanvasPixmap.DrawPixel(x,y,c);
}

void LCanvas::FastDrawLine(int x1,int y1,int x2,int y2){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::DrawLine(x1,y1,x2,y2);
}

void LCanvas::DrawLine(int x1,int y1,int x2,int y2){
  // Call the fast version:
  FastDrawLine(x1,y1,x2,y2);
  
  // Update CanvasPixmap:
  CanvasPixmap.DrawLine(x1,y1,x2,y2);
}

void LCanvas::FastDrawLine(int x1,int y1,int x2,int y2,const LColor c){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::DrawLine(x1,y1,x2,y2,c);
}

void LCanvas::DrawLine(int x1,int y1,int x2,int y2,const LColor c){
  // Call the fast version:
  FastDrawLine(x1,y1,x2,y2,c);

  // Update CanvasPixmap:
  CanvasPixmap.DrawLine(x1,y1,x2,y2,c);
}

void LCanvas::FastDrawFullRectangle(int x1,int y1,int x2,int y2){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::DrawFullRectangle(x1,y1,x2,y2);
}

void LCanvas::DrawFullRectangle(int x1,int y1,int x2,int y2){
  // Call the fast version:
  FastDrawFullRectangle(x1,y1,x2,y2);
  
  // Update CanvasPixmap:
  CanvasPixmap.DrawFullRectangle(x1,y1,x2,y2);
}

void LCanvas::FastDrawFullRectangle(int x1,int y1,int x2,int y2,const LColor c){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::DrawFullRectangle(x1,y1,x2,y2,c);
}

void LCanvas::DrawFullRectangle(int x1,int y1,int x2,int y2,const LColor c){
  // Call the fast version:
  FastDrawFullRectangle(x1,y1,x2,y2,c);
  
  // Update CanvasPixmap:
  CanvasPixmap.DrawFullRectangle(x1,y1,x2,y2,c);
}

void LCanvas::FastDrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::DrawFullTriangle(x1,y1,x2,y2,x3,y3);
}

void LCanvas::DrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3){
  // Call the fast version:
  DrawFullTriangle(x1,y1,x2,y2,x3,y3);
  
  // Update CanvasPixmap:
  CanvasPixmap.DrawFullTriangle(x1,y1,x2,y2,x3,y3);
}

void LCanvas::FastDrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3,const LColor c){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::DrawFullTriangle(x1,y1,x2,y2,x3,y3,c);
}

void LCanvas::DrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3,const LColor c){
  // Call the fast version:
  DrawFullTriangle(x1,y1,x2,y2,x3,y3,c);
  
  // Update CanvasPixmap:
  CanvasPixmap.DrawFullTriangle(x1,y1,x2,y2,x3,y3,c);
}

void LCanvas::FastSetColor(const LColor c){
  // Dummy method calling the LDrawable parent class version:
  LDrawable::SetColor(c);
}

void LCanvas::SetColor(const LColor c){
  // Call the fast version:
  FastSetColor(c);

  // Update CanvasPixmap:
  CanvasPixmap.SetColor(c);
}

const LColor LCanvas::GetColor() const{
  // Dummy method calling the LDrawable parent class version:
  return LDrawable::GetColor();
}

void LCanvas::FastCopyAreaFrom(int x1dest,int y1dest,
                               LPixmap &source,int x1,int y1,int x2,int y2){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::CopyAreaFrom(x1dest,y1dest,source,x1,y1,x2,y2);
}

void LCanvas::FastCopyAreaFrom(int x1dest,int y1dest,
                               LCanvas &source,int x1,int y1,int x2,int y2){
  // Dummy method calling the LDrawable parent class version:
  if(drawable=XtWindow(SysDep.area))
    LDrawable::CopyAreaFrom(x1dest,y1dest,source.CanvasPixmap,x1,y1,x2,y2);
}

void LCanvas::CopyAreaFrom(int x1dest,int y1dest,
                           LPixmap &source,int x1,int y1,int x2,int y2){
  // Call the 'fast' version:
  FastCopyAreaFrom(x1dest,y1dest,source,x1,y1,x2,y2);

  // Update the CanvasPixmap:
  CanvasPixmap.CopyAreaFrom(x1dest,y1dest,source,x1,y1,x2,y2);
}

void LCanvas::CopyAreaFrom(int x1dest,int y1dest,
                           LCanvas &source,int x1,int y1,int x2,int y2){
  // Call the 'fast' version:
  FastCopyAreaFrom(x1dest,y1dest,source,x1,y1,x2,y2);

  // Update the CanvasPixmap:
  CanvasPixmap.CopyAreaFrom(x1dest,y1dest,source,x1,y1,x2,y2);
}

