//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "../../LMenu.h"
#include "../../LAttributes.h"
#include "../../LTypes.h"

INDEX menuhashfunction(Lstring s){
  int r=0, // Value to be returned
      i=0; // Loop counter

  for(i=0;i<s.getdim();i++)
    r+=s[i];
  return r;
}

// Constructor used to initialize hash tables
SysDepMenuData::SysDepMenuData():callbacks(menuhashfunction),
                                 widgets(menuhashfunction){
  // Empty body
}

// General Xt callback used to call the real LTakeshi callback
static void UniversalLMenuCallBack(Widget w,XtPointer callData,
                                   XtPointer clientData){
  // This dummy callback simply calls the real LCallBack function, which
  // can be found through callData, which in its turn contains a pointer to the
  // LTakeshi widget, and through the resources of the Xt widget:
  LMenu &thismenu=*((LMenu*)callData);

  // Find the LTakeshi callback to be called: look at the hash table in
  // thismenu with the right entry name.
  // First get the entry name:
  char *entryname;
  XtVaGetValues(w,
                XtNlabel, &entryname,
                0);

  // Now find the LTakeshi callback; it's guaranteed to exist because of
  // the way LMenu::Apply() is written:
  LCallBack callback=thismenu.SysDep.callbacks[entryname];

  // And finally call the LTakeshi callback, if it's not null:
  if(callback!=0)
    callback(thismenu,LVoidEvent);
}

// Internal counter used to name menus
static long lastmenu=0;

void LMenu::SystemDependantInit(){
  // Athena uses a strange strategy to bind the menu shell to the menu
  // button: instead of the widget, it wants the _name_; a good way to find
  // unique name is assigning a number to every instance:
  strncpy(SysDep.menuname,("LMenu"+Lstring(lastmenu++)).ToCharStar(),10);

  // I cannot set the background color with fallback resources as usual,
  // because I don't know the widget name: so I copy the value from the
  // parent window skeleton to menuwidget.
  Pixel backgroundcolor;
  XtVaGetValues(parent->SysDep.area,
                XtNbackground,      &backgroundcolor,
                0);

  // Create menu widget
  SysDep.menuwidget=XtVaCreatePopupShell(SysDep.menuname,
                    simpleMenuWidgetClass,
                    parent->SysDepAsWindow.shell,
                    XtNbackground,	backgroundcolor,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                    0);

  // Create menu button widget, which activates SysDep.menuwidget
  SysDep.menubuttonwidget=XtVaCreateManagedWidget("LMenuButton",
                    menuButtonWidgetClass,parent->SysDepAsWindow.menubar,
                    XtNmenuName,        SysDep.menuname,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                    0);
}

void LMenu::SystemDependantDone(){
}

void LMenu::Apply(){
  LMenuList *Pointer1=modifications,*Pointer2; // Pointers used to scan the
                                               // list
  // Change the menu caption if LAText changed
  if(TestAndSet(LAText))
    XtVaSetValues(SysDep.menubuttonwidget,
                  XtNlabel,	GetString(LAText).ToCharStar(),
                  0);

  // Add new menu entries if needed
  while(Pointer1!=0){
    if(strcmp(Pointer1->ItemName,"-")){ // If this entry is not a separator...
      Widget entry= XtVaCreateManagedWidget("LMenuEntry", // ...add an entry
            smeBSBObjectClass,	SysDep.menuwidget,
            XtNlabel,		Pointer1->ItemName,
            XtNsensitive,	TRUE,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
            0);

      // Add a callback (I assume this is the only one): the parameter is
      // needed to find the right argument to the LTakeshi callback, which
      // needs a reference to the LTakeshi widget itself.
      XtAddCallback(entry,XtNcallback,UniversalLMenuCallBack,this);

      // Add entries to the callbacks and widgets hash tables
      SysDep.callbacks.insert(Lstring(Pointer1->ItemName),Pointer1->
                                                          ItemCallBack);
      SysDep.widgets.insert(Lstring(Pointer1->ItemName),entry);
    }
    else                               // else...
      XtVaCreateManagedWidget("LMenuSeparator", // ...add a separator
            smeLineObjectClass,	SysDep.menuwidget,
            XtNsensitive,		TRUE,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
            0);

    // Make Pointer1 advance and free the storage which it pointed
    Pointer1=(Pointer2=Pointer1)->Next;
    free(Pointer2);
  }

  // The list is empty one more time: initialize again pointers
  modifications=last=0;
}

void LMenu::Enable(char *entry){
  // If the requested menu entry exists then make sensitive its Athena widget
  if(SysDep.widgets.exists(entry))
    XtVaSetValues(SysDep.widgets[entry],
                  XtNsensitive,	True,
                  0);
  else // else issue a warning
    cerr<<"LMenu::Enable(): The menu entry "<<GetString(LAText)
        <<"|"<<entry<<" does not exist.\n";
}

void LMenu::Disable(char *entry){
  // If the requested menu entry exists then make unsensitive its Athena
  // widget
  if(SysDep.widgets.exists(entry))
    XtVaSetValues(SysDep.widgets[entry],
                  XtNsensitive,	False,
                  0);
  else // else issue a warning
    cerr<<"LMenu::Disable(): The menu entry "<<
        GetString(LAText)<<"|"<<entry<<"does not exist.\n";
}



