//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


extern "C"{
#include <stdio.h>
}

#include "LTakeshi.h"
#include <iostream.h>


LWindow w;
LInputBox it1(w);

void BeepCallBack(LWidget &itself,const LEvent event){
  cout<<"Beep\n\a";
}

void TestCallBack(LWidget &itself,const LEvent event){
  cout<<"From the callback: ("<<event.MouseEvent.x<<","<<event.MouseEvent.y<<")\n";
}

void ChoiceCallBack(LWidget &itself,const LEvent event){
  ChoiceDialog("Is Takeshi a great tool?",
               "Yes (beep)",	BeepCallBack,
               "Oh, yes",	0,
               "Of course",	0,
               "No doubt",	0,
               "Sure",		0);
}

void MessageCallBack(LWidget &itself,const LEvent event){
  ShowMessage("It really works, incredible! [To do: make autowrap]");
}

void EchoCallBack(LWidget &itself,const LEvent event){
  Lstring s;
    it1.Get(LAText,	&s,
            LAEnd);
  cerr<<"The it1 LInputBox widget contains the \""<<s<<"\" string \n";
}

void ExitCallBack(LWidget &itself,const LEvent event){
  cerr<<"\'Exit\' was pushed\n";
  exit(EXIT_SUCCESS);
}

void PhotoCallBack(LWidget &itself,const LEvent event){
  cerr<<"\'My photo bitmap\' was pushed\n";
}

void InputBoxCallBack(LWidget &itself,const LEvent event){
  Lstring name=itself.GetString(LAName);
  cerr<<"The text of the input text box named \""<<name
      <<"\" has been modified\n";
}

LWindow    square;
LTextLabel stl1(square);
LTextLabel stl2(square);
LTextLabel stla(square);
LInputBox  sib(square);
LButton    sb(square);

void SquareCloseCallBack(LWidget &itself,const LEvent event){
  square.Hide();
}

void SquareUpdateCallBack(LWidget &itself,const LEvent event){
  square.Set(LAStatusBarText,	"Wait...",
             LAEnd);

  Lstring side;
  sib.Get(LAText,	&side,
          LAEnd);

  stla.Set(LAText,	Lstring(side.ToLong()*side.ToLong()).ToCharStar(),
           LAEnd);

  square.Set(LAStatusBarText,	"Ready",
             LAEnd);
}

void SquareCallBack(LWidget &itself,const LEvent event){
  square.Set(LAW,	200,
             LAH,	100,
             LATitle,	"Area of a square",
             LAModal,	true,
             LAEnd);

  stl1.Set(LAText,	"Side:",
           LAX,		20,
           LAY,		20,
           LAEnd);

  stl2.Set(LAText,	"Area:",
           LAX,		100,
           LAY,		20,
           LAEnd);

  stla.Set(LAText,	"0",
           LAX,		100,
           LAY,		40,
           LAEnd);

  sib.Set(LAOnChange,   SquareUpdateCallBack,
          LAText,	"0",
          LAMax,	4,
          LAX,		20,
          LAY,		40,
          LAW,		60,
          LAEnd);

  sb.Set(LAX,		60,
         LAY,		70,
         LAW,		80,
         LAText,	"Close",
         LAEffect,	SquareCloseCallBack,
         LAEnd);

  square.Show();
}

void ChangeCallBack(LWidget &w,const LEvent event){
  cerr<<"Change\n";
}

void ScrollCallBack(LWidget &w,const LEvent event){
  cerr<<"Scroll\n";
}

LCanvas c(w);
LCanvas c2(w);

#define CASO(n) long(float(rand())/RAND_MAX*n)

void DrawCallBack(LWidget &w,const LEvent event){
  int width,height,x1,y1;

  c.Get(LAX,  &x1,
        LAY,  &y1,
        LAW,  &width,
        LAH,  &height,
        LAEnd);

  int x,y;
  for(y=0;y<height;y++)
    for(x=0;x<width;x++){
      c2.SetColor(LColor(x*MaxLColorComponent/width,
                         y*MaxLColorComponent/height,
                         0
                        )
                 );
      c2.DrawLine(x,y,y%width,x%height);
      c2.DrawLine(width-x,height-y,width-y%width,height-x%height);
    }
  c.CopyAreaFrom(0,0,c2,0,0,width,height);

  LPixmap p;
  p.Resize(width,height);
  p.CopyAreaFrom(0,0,c,0,0,width,height);
  c.CopyAreaFrom(20,20,p,0,0,30,30);
}

void ClearCallBack(LWidget &w){
  c.Clear();
  c2.Clear();
}

int main(){
  w.Set(LAX,            200,
        LAY,            100,
        LAW,            400,
        LAH,            375,
        LATitle,        "This is a LTakeshi window",
        LAStatusBarText,"This is the status bar",
        LAEnd);

  LTextLabel t1(w);
  t1.Set(LAX,     30,
         LAY,     20,
         LAText,  "Some text...",
         LAEnd);

  LTextLabel t2(w);
  t2.Set(LAX,     50,
         LAY,     40,
         LAText,  "...more text",
         LAEnd);

  LButton b1(w);
  b1.Set(LAX,     30,
         LAY,     60,
         LAText,  "Choice",
         LAEffect,ChoiceCallBack,
         LAEnd);

  LButton bp(w);
  bp.Set(LAX,     20,
         LAY,     90,
         LAText,  "Message",
         LAEffect,MessageCallBack,
         LAEnd);

  LButton b2(w);
  b2.Set(LAX,     120,
         LAY,     90,
         LAText,  "Echo",
         LAEffect,EchoCallBack,
         LAEnd);

  LButton b3(w);
  b3.Set(LAX,     280,
         LAY,     5,
         LAText,  "Exit",
         LAEffect,ExitCallBack,
         LAEnd);

  LButton b4(w);
  b4.Set(LAX,     50,
         LAY,     120,
         LAText,  "Beep",
         LAEffect,BeepCallBack,
         LAEnd);

  LButton b5(w);
  b5.Set(LAX,     280,
         LAY,     35,
         LAImage, "foto.xbm",
         LAEffect,PhotoCallBack,
         LAEnd);

  LButton b6(w);
  b6.Set(LAX,     180,
         LAY,     120,
         LAText,  "Square",
         LAEffect,SquareCallBack,
         LAEnd);
  
  it1.Set(LAX,		50,
          LAY,		150,
          LAText,	"abcdef",
          LAMax,	8,
          LAOnChange,   InputBoxCallBack,
          LAEnd);

  LMenu m1(w,"Buttons");
  m1.Insert("Echo",	   EchoCallBack,
            "Beep",	   BeepCallBack,
            "-",
            "Print...",	MessageCallBack,
            "-",
            "Exit",     ExitCallBack,
            LAEnd);
  m1.Disable("Print...");

  LMenu m2(w,"Dialogs");
  m2.Insert("Message...", MessageCallBack,
            "Choice...",  ChoiceCallBack,
            "Square...",  SquareCallBack,
            LAEnd);

  LScrollBar sb(w);
  sb.Set(LAX,		150,
         LAH,		70,
         LAOnChange,	ChangeCallBack,
         LAOnScroll,	ScrollCallBack,
         LAEnd);

  c.Set(LAX,            175,
        LAY,            5,
        LAW,            100,
        LAH,            70,
        LAOnMouseClick, TestCallBack,
        LAOnMouseMove,  TestCallBack,
        LAEnd);

  c2.Set(LAX,      175,
         LAY,      155,
         LAW,      100,
         LAH,      70,
         LAEnd);

  LButton bdraw(w);
  bdraw.Set(LAX,     280,
            LAY,     125,
            LAText,  "Draw",
            LAEffect,DrawCallBack,
            LAEnd);

  LButton bclear(w);
  bclear.Set(LAX,     280,
             LAY,     150,
             LAText,  "Clear",
             LAEffect,ClearCallBack,
             LAEnd);

  w.Show();

  TakeshiMainLoop();
  return 0;
}

