#ifndef _LBLIST_H
#define _LBLIST_H

#include <iostream.h>
#include "Lgen/Lgen.h"

extern "C"{
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
}

template <class T> struct blistelement{
 T c;
 blistelement<T> *pred,*next;
};

template <class T,int SELS> class Lblist{
 blistelement<T> *head,*tail,*last,*s[SELS];
 INDEX dim;
public:
 Lblist();
 ~Lblist();

 //primary functions
 inline bool isempty();
 void inserthead(T);
 void inserttail(T);
 int removehead();
 int removetail();
 int remove(T);
 int removesatisfying(bool(*f)(T&));
 T* searchsatisfying(bool(*)(T&));
 T* searchsatisfying(bool(*)(T));
 T& accesshead();
 T& accesstail();

 //selection-related functions
 inline T& accessselected(int);
 void unselect(int);
 void selecthead(int);
 void selecttail(int);
 inline int selectnext(int);
 inline int selectpred(int);
 int selectnextcycle(int);
 int selectpredcycle(int);
 int removeselected(int);
 int insertbeforeselected(int,T);
 int insertafterselected(int,T);
 T* select(int,T);
 T* selectsatisfying(int,bool(*)(T&));
 T* selectsatisfying(int,bool(*)(T));
 bool existsselection(int);
 bool selectionishead(int);
 bool selectionistail(int);

 //index-releted functions
 int selectindex(int,INDEX);
 inline INDEX getdim();
 T& operator[](INDEX);

 //stream-related functions
 friend ostream& operator<< <>(ostream&,Lblist<T,SELS>&);

 //sorting
 void sort(bool(*)(T*,T*));
};
  //int methods' return-values are:
  //
  //  / 0           success
  // {
  //  \ -1          some kind of non-fatal error: where it makes sense,
  //                command is however executed

template <class T,int SELS> T* Lblist<T,SELS>::searchsatisfying(bool(*f)(T)){
 last=head;
 while(last){
  if(f(last->c))
   return &(last->c);
  last=last->next;
 }
 return EXIT_SUCCESS;
}

template <class T,int SELS> T* Lblist<T,SELS>::searchsatisfying(bool(*f)(T&)){
 last=head;
 while(last){
  if(f(last->c))
   return &(last->c);
  last=last->next;
 }
 return 0;
}

template <class T,int SELS> int Lblist<T,SELS>::removesatisfying(bool(*f)(T&)){
 last=head;
 int r=EXIT_FAILURE;

 while(last){
  if(f(last->c)){
   if(last->pred)
    last->pred->next=last->next;
   else
    head=last->next;
   if(last->next)
    last->next->pred=last->pred;
   else
    tail=last->pred;
   dim--;
   r=EXIT_SUCCESS;
  }
  last=last->next;
 }
 return r;
}

template <class T,int SELS> int Lblist<T,SELS>::remove(T x){
 last=head;
 int r=EXIT_FAILURE;

 while(last){
  if(last->c==x){
   if(last->pred)
    last->pred->next=last->next;
   else
    head=last->next;
   if(last->next)
    last->next->pred=last->pred;
   else
    tail=last->pred;
   dim--;
   r=EXIT_SUCCESS;
  }
  last=last->next;
 }
 return r;
}

template <class T,int SELS> INDEX Lblist<T,SELS>::getdim(){
 return dim;
}

template <class T,int SELS> void Lblist<T,SELS>::unselect(int e){
 s[e]=0;
 return;
}

template <class T,int SELS> T* Lblist<T,SELS>::selectsatisfying(int e,bool(*f)(T)){
 last=head;
 while(last){
  if(f(last->c)){
   s[e]=last;
   return &(s[e]->c);
  }
  last=last->next;
 }
 return 0;
}

template <class T,int SELS> T* Lblist<T,SELS>::selectsatisfying(int e,bool(*f)(T&)){
 last=head;
 while(last){
  if(f(last->c)){
   s[e]=last;
   return &(s[e]->c);
  }
  last=last->next;
 }
 return 0;
}

template <class T,int SELS> bool Lblist<T,SELS>::existsselection(int e){
 return s[e]!=0;
}

template <class T,int SELS> T* Lblist<T,SELS>::select(int e,T x){
 last=head;

 while(last){
  if(last->c==x){
   s[e]=last;
   return &(s[e]->c);
  }
  last=last->next;
 }
 return 0;
}

template <class T,int SELS> void Lblist<T,SELS>::selecthead(int e){
 s[e]=head;
 return;
}

template <class T,int SELS> void Lblist<T,SELS>::selecttail(int e){
 s[e]=tail;
 return;
}

template <class T,int SELS> int Lblist<T,SELS>::selectnext(int e){
 if((!s[e])||(s[e]==tail)||(isempty()))
  return EXIT_FAILURE;
 else{
  s[e]=s[e]->next;
  return EXIT_SUCCESS;
 }
}

template <class T,int SELS> int Lblist<T,SELS>::selectnextcycle(int e){
 if(selectnext(e)==EXIT_SUCCESS)
  return EXIT_SUCCESS;
 else{
  if(head){
   selecthead(e);
   return EXIT_SUCCESS;
  }
  else
   return EXIT_FAILURE;
 }
}

template <class T,int SELS> int Lblist<T,SELS>::selectpredcycle(int e){
 if(selectpred(e)==EXIT_SUCCESS)
  return EXIT_SUCCESS;
 else{
  if(tail){
   selecttail(e);
   return EXIT_SUCCESS;
  }
  else
   return EXIT_FAILURE;
 }
}

template <class T,int SELS> int Lblist<T,SELS>::selectpred(int e){
 if((!s[e])||(s[e]==head)||(isempty()))
  return EXIT_FAILURE;
 else{
  s[e]=s[e]->pred;
  return EXIT_SUCCESS;
 }
}

template <class T,int SELS> T& Lblist<T,SELS>::accessselected(int e){
 if(s[e])
  return s[e]->c;
 else
  cerr<<"No blistelement is selected in selection #"<<e;
  fatal("Lblist<T,SELS>: attempt to access selected blistelement when"
        " no selection is active");
}

template <class T,int SELS> bool Lblist<T,SELS>::selectionishead(int e){
 return s[e]==head;
}

template <class T,int SELS> bool Lblist<T,SELS>::selectionistail(int e){
 return s[e]==tail;
}

template <class T,int SELS> T& Lblist<T,SELS>::accesshead(){
 if(isempty()){
  fatal("Lblist<T,SELS>: access to head of empty Lblist");
  return *(new T);//just to avoid warnings from GCC --pedantic
 }else
  return(head->c);
}

template <class T,int SELS> T& Lblist<T,SELS>::accesstail(){
 if(isempty()){
  fatal("Lblist<T,SELS>: access to tail of empty Lblist");
  return *(new T);//just to avoid warnings from GCC --pedantic
 }else
  return(tail->c);
}

template <class T,int SELS> Lblist<T,SELS>::Lblist(){
 head=tail=last=0;
 int ct;
 for(ct=0;ct<SELS;ct++)
  s[ct]=0;
 dim=(INDEX)0;
}

template <class T,int SELS> Lblist<T,SELS>::~Lblist(){
 while(!isempty()){
  last=head->next;
  delete(head);
  head=last;
 }
}

template <class T,int SELS> bool Lblist<T,SELS>::isempty(){
 return(!head);
}

template <class T,int SELS> void Lblist<T,SELS>::inserthead(T x){
 last=new(blistelement<T>);
 if(isempty())
  tail=last;
 last->c=x;
 last->next=head;
 last->pred=0;
 if(last->next)
  last->next->pred=last;
 head=last;
 dim++;
 return;
}

template <class T,int SELS> void Lblist<T,SELS>::inserttail(T x){
 last=new(blistelement<T>);
 if(isempty())
  head=last;
 last->c=x;
 last->next=0;
 last->pred=tail;
 if(last->pred)
  last->pred->next=last;
 tail=last;
 dim++;
 return;
}

template <class T,int SELS> int Lblist<T,SELS>::removehead(){
 if(isempty())
  return EXIT_FAILURE;
 else{
  int e;
  for(e=0;e<SELS;e++)
   if(head==s[e])
    selectnext(e);
  blistelement<T> *p=head->next;
  delete head;
  head=p;
  if(head)
   head->pred=0;
  else
   tail=0;
  dim--;
  return EXIT_SUCCESS;
 }
}

template <class T,int SELS> int Lblist<T,SELS>::removetail(){
 if(isempty())
  return EXIT_FAILURE;
 else{
  int e;
  for(e=0;e<SELS;e++)
   if(tail==s[e])
    selectpred(e);
  last=tail->pred;
  delete tail;
  tail=last;
  if(tail)
   tail->next=0;
  else
   head=0;
  dim--;
  return EXIT_SUCCESS;
 }
}

template <class T,int SELS> ostream& operator<<(ostream& S,Lblist<T,SELS>& L){
 L.last=L.head;
 S<<'(';
 while(L.last){
  if(L.last!=L.head)
   S<<' ';
   S<<L.last->c;
  L.last=L.last->next;
 }
 S<<')';
 return S;
}

template <class T,int SELS> int Lblist<T,SELS>::removeselected(int e){
 if(s[e]){
  if(s[e]->next)
   s[e]->next->pred=s[e]->pred;
  if(s[e]->pred)
   s[e]->pred->next=s[e]->next;
  last=s[e];
  if(last->next)
   s[e]=last->next;
  else
   s[e]=last->pred;
  if(!(last->pred)&&(!(last->next)))
   head=tail=0;
  delete last;
  if(s[e]){
   if(!(s[e]->next))
    tail=s[e];
   if(!(s[e]->pred))
    head=s[e];
  }
  dim--;
  return EXIT_SUCCESS;
 }
 else
  return EXIT_FAILURE;
}

template <class T,int SELS> int Lblist<T,SELS>::insertbeforeselected(int e,T x){
 if(!s[e])
  return EXIT_FAILURE;
 else{
  last=new(blistelement<T>);
  last->c=x;
  last->next=s[e];
  if(s[e]->pred)
   s[e]->pred->next=last;
  else
   head=last;
  last->pred=s[e]->pred;
  s[e]->pred=last;
  dim++;
  return EXIT_SUCCESS;
 }
}

template <class T,int SELS> int Lblist<T,SELS>::insertafterselected(int e,T x){
 if(!s[e])
  return EXIT_FAILURE;
 else{
  last=new(blistelement<T>);
  last->c=x;
  last->pred=s[e];
  if(s[e]->next)
   s[e]->next->pred=last;
  else
   tail=last;
  last->next=s[e]->next;
  s[e]->next=last;
  dim++;
  return EXIT_SUCCESS;
 }
}

template <class T,int SELS> int Lblist<T,SELS>::selectindex(int e,INDEX i){
 if((i<0)||(i>=dim))
  return EXIT_FAILURE;
 else{
  if(i<=(dim/2)){
   INDEX ct;

   last=head;
   for(ct=0;ct<i;ct++){
    last=last->next;
   }
   s[e]=last;
   return EXIT_SUCCESS;
  }
  else{
   INDEX ct;

   last=tail;
   for(ct=dim-1;ct>i;ct--){
    last=last->pred;
   }
   s[e]=last;
   return EXIT_SUCCESS;
  }
 }
}

template <class T,int SELS> T& Lblist<T,SELS>::operator[](INDEX i){
 if((i<0)||(i>=dim))
  fatal("Lblist<T,SELS>: blistelement is out of range");
 else{
  if(i<=(dim/2)){
   INDEX ct;

   last=head;
   for(ct=0;ct<i;ct++){
    last=last->next;
   }
   return last->c;
  }
  else{
   INDEX ct;

   last=tail;
   for(ct=dim-1;ct>i;ct--){
    last=last->pred;
   }
   return last->c;
  }
 }
}

template <class T,int SELS> void Lblist<T,SELS>::sort(bool(*comp)(T*,T*)){
 T *buffer=(T*)xmalloc(dim*sizeof(T));
 INDEX ct=0;

 last=head;
 while(last){
  buffer[ct++]=last->c;
  last=last->next;
 }
 qsort((void*)buffer,(size_t)dim,sizeof(T),(int(*)(const void*,
  const void*))comp);
 ct=0;
 last=head;
 while(last){
  last->c=buffer[ct++];
  last=last->next;
 }
 free(buffer);
 return;
}
#endif






