//This file is part of the Lhash v1.0 library written in -*- C++ -*-

//Lhash v1.0: a simple hash table class
//Copyright (C) Luca Saiu, 1999-2000

//This library is free software; you can redistribute it and/or modify it
//under the terms of the GNU Library General Public License version 2
//as published by the Free Software Foundation.

//This library is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Library General Public License for more details.

//You should have received a copy of the GNU Library General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


//Luca Saiu
//------------------------------------------
//e-mail:      luca78@dada.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


// To do: rewrite from scratch in a better style

#ifndef _LHASH_H
#define _LHASH_H

#include "Lgen/Lgen.h"
#include <iostream.h>

extern "C"{
#include <stdlib.h>
}

//#define DEBUG

template <class D,class K> struct element{
  K key;
  D data;
  element<D,K>* next;
  element(){
    next=0;
  }
};

template <class D,class K,INDEX m> class Lhash{
private:
  element<D,K> **table,*temp1,*temp2;
  bool b;
  INDEX(*h) (const K k);
public:
  Lhash(INDEX(*)(const K k)); // the hash funcion is required as a parameter
                              // it can return any integer value; the '% m'
                              // will be appended automatically

  Lhash(const Lhash&);        // Copy constructor
  ~Lhash();
  int insert(const K,const D);
  int remove(const K);
  bool IsEmpty() const;
  inline D* access(const K);
  inline const D* access(const K) const;
  inline D& operator[](const K);
  inline const D& operator[](const K) const;
  bool exists(const K) const;
  void Dump(ostream &s=cerr) const;
  void Empty();              // deletes every element
};

// Implementation:
template <class D,class K,INDEX m> Lhash<D,K,m>::Lhash(const Lhash<D,K,m>
  &x){
  // Initialize
  table=(element<D,K>**)malloc(sizeof(element<D,K>*)*m);

  // Copy the hash function
  h=x.h;

  // Now copy every list
  INDEX ct;
  element<D,K> *thisp,*xp;

  for(ct=0;ct<m;ct++){
    if(x.table[ct]==0) // If the ct-th list is empty...
      table[ct]=0;     // ...make empty also this copy
    else{
      // Copy the ct-th list
      table[ct]=new element<D,K>;
      *(table[ct])=*(x.table[ct]);
      xp=x.table[ct]->next;
      thisp=table[ct];
      while(xp!=0){
        thisp->next=new element<D,K>;
        *(thisp->next)=*xp;
        thisp=thisp->next;
        xp=xp->next;
      }
      thisp->next=0;
      // Now the ct-th list has been copied
    } // else{
  }//for(ct=0;ct<m;ct++){
}

template <class D,class K,INDEX m> bool Lhash<D,K,m>::IsEmpty() const{
  int i;
  // Control if every element of table points to 0: is there is at least
  // one pointing to something else return false, else return true
  for(i=0;i<m;i++)
    if(table[i]!=0)
      return false;
  // At this point I can be sure there are no elements: the hash table is
  // empty
  return true;
}

template <class D,class K,INDEX m> Lhash<D,K,m>::Lhash(INDEX(*h_)(K k)){
  if(!m)
    fatal("Lhash<D,K,m>: m must be greater than 0");
  table=(element<D,K>**)malloc(sizeof(element<D,K>*)*m);
  INDEX ct;
  for(ct=0;ct<m;ct++)
    table[ct]=0;
  h=h_;
#ifdef DEBUG
  cerr<<"HashTable has been created\n";
#endif
}

template <class D,class K,INDEX m> void Lhash<D,K,m>::Empty(){
  INDEX ct;
  for(ct=0;ct<m;ct++){
#ifdef DEBUG
    cerr<<"\nAddress "<<ct<<": ";
#endif
    temp1=table[ct];
    while(temp1!=0){
      temp2=temp1->next;
#ifdef DEBUG
      cerr<<" "<<(temp1->key)<<": "<<(temp1->data)<<", ";
#endif
      delete(temp1);
      temp1=temp2;
    }// while(
    table[ct]=0;
  }// for(
}

template <class D,class K,INDEX m> Lhash<D,K,m>::~Lhash(){
 // Empty the hash table:
 Empty();

 // Free storage for the table of pointers:
 free(table);
#ifdef DEBUG
 cerr<<"\nHashTable has been destroyed\n";
#endif
}

template <class D,class K,INDEX m> int Lhash<D,K,m>::insert(const K key, const D data){
  b=false;
  INDEX address=(h(key)%m);
  temp2=table[address];
  while(temp2){
    if(temp2->key==key)
    b=true;
    temp2=temp2->next;
  }
  if(b==false){
    temp1=new element<D,K>;
    temp1->key=key;
    temp1->data=data;
    temp1->next=table[address];
    table[address]=temp1;
#ifdef DEBUG
    cerr<<"element with \'"<<key<<"\' key has been inserted at "<<address<<endl;
#endif
    return EXIT_SUCCESS;
  }
  else{ // b==true
    operator[](key)=data;
    return EXIT_FAILURE;
  }
}

template <class D,class K,INDEX m> int Lhash<D,K,m>::remove(const K key){
 INDEX address=h(key)%m;
 if(temp1=table[address])
  if(temp1->key==key){
   table[address]=temp1->next;
   delete(temp1);
#ifdef DEBUG
   cerr<<"Element "<<key<<" has been deleted"<<endl;
#endif
   return EXIT_SUCCESS;
  }
 while(temp1){
  temp2=temp1;
  temp1=temp1->next;
  //Now temp1==temp2->next
  if(temp1)
   if(temp1->key==key){
    //remove temp1
    temp2->next=temp1->next;
    delete temp1;
#ifdef DEBUG
    cerr<<"Element \'"<<key<<"\' has been deleted"<<endl;
#endif
    return EXIT_SUCCESS;
   }
 }
 return EXIT_FAILURE;
}

template <class D,class K,INDEX m> D* Lhash<D,K,m>::access(const K key){
 temp1=table[h(key)%m];
 while(temp1){
  if(temp1->key==key)
   return &(temp1->data);
  temp1=temp1->next;
 }
 return 0;
}

template <class D,class K,INDEX m> const D* Lhash<D,K,m>::access(const K key) const{
 element<D,K> *temp1=table[h(key)%m];
 while(temp1){
  if(temp1->key==key)
   return &(temp1->data);
  temp1=temp1->next;
 }
 return 0;
}

template <class D,class K,INDEX m> D& Lhash<D,K,m>::operator[](const K key){
 D *r=access(key);
 if(r)
  return *r;
 else{
  fatal("Lhash<D,K,m>: element with selected key does not exist");
  return operator[](key); //Just to avoid a compiler warning
 }
}

template <class D,class K,INDEX m> const D& Lhash<D,K,m>::operator[](const K key) const{
 D *r=access(key);
 if(r)
  return *r;
 else{
  fatal("Lhash<D,K,m>: element with selected key does not exist");
  return operator[](key); //Just to avoid a compiler warning
 }
}

template <class D,class K,INDEX m> bool Lhash<D,K,m>::exists(const K key) const{
 INDEX address=(h(key)%m);
 element<D,K> *temp1=table[address];
 while(temp1){
  if(temp1->key==key)
   return true;
  temp1=temp1->next;
 }
 return false;
}

template <class D,class K,INDEX m> void Lhash<D,K,m>::Dump(ostream& s) const{
  int i;
  element<D,K> *tmp;

  for(i=0;i<m;i++)   // For every list...
    if(table[i]!=0){ // ...if the list is not empty...
       // For each element in list, write on the s stream <KEY> -> <DATA>\n
       tmp=table[i];
       do{
         s<<(tmp->key)<<" -> "<<(tmp->data)<<endl; // write a line
         tmp=tmp->next;
       }while(tmp!=0);
    }
}

#endif

