#ifndef _LKTREE_H
#define _LKTREE_H

#include "Lgen/Lgen.h"

template <int k,class T> class Lktree{
 unsigned depth;//also an unsigned char would have been enough...
 Lktree<k,T> *parent,*sons[k];
 T c;
public:
 Lktree();
 Lktree(Lktree<k,T>&);
 Lktree(T x);
 ~Lktree();
 inline T& access();
 inline Lktree<k,T> *accessparent();
 inline Lktree<k,T> *accessson(int);
 inline BOOL isleaf();
 inline unsigned getdepth();
 int createson(int,T);
 int removeson(int);
 friend ostream& operator<< <>(ostream&,Lktree<k,T>&);
};

template <int k,class T> T& Lktree<k,T>::access(){
 return c;
}

template <int k,class T> Lktree<k,T>::Lktree(T x){
 parent=0;
 depth=0;
 int ct;
 for(ct=0;ct<k;ct++)
  sons[ct]=0;
 c=x;
}

template <int k,class T> Lktree<k,T>::Lktree(){
 parent=0;
 depth=0;
 int ct;
 for(ct=0;ct<k;ct++)
  sons[ct]=0;
}

template <int k,class T> Lktree<k,T>::Lktree(Lktree<k,T> &T){
 c=T.c;
 depth=T.depth;
 int ct;
 parent=0;
 for(ct=0;ct<k;ct++)
  if(T.accessson(ct)){
   sons[ct]=new Lktree(*T.sons[ct]);
   sons[ct]->parent=this;
  }
  else
   sons[ct]=0;
}

template <int k,class T> Lktree<k,T>::~Lktree(){
 int ct;
// cerr<<"Destroying \'"<<c<<"\'..."<<endl;
 for(ct=0;ct<k;ct++)
  if(sons[ct])
   delete sons[ct];
 if(parent){
   for(ct=0;ct<k;ct++)
     if(parent->sons[ct]==this){
       parent->sons[ct]=0;
       break;
     }
 }
// cerr<<"...done"<<endl;

}

template <int k,class T> int Lktree<k,T>::createson(int i,T x){
 if((i<0)||(i>=k))
  return EXIT_FAILURE;
 int r=EXIT_FAILURE;
 if(!sons[i]){
  sons[i]=new Lktree<k,T>;
  sons[i]->parent=this;
  sons[i]->depth=depth+1;
  r=EXIT_SUCCESS;
 }
 sons[i]->c=x;
 return r;
}

template <int k,class T> int Lktree<k,T>::removeson(int i){
 if((i<0)||(i>=k))
  return EXIT_FAILURE;
 if(sons[i]){
  delete sons[i];
  sons[i]=0;
  return EXIT_SUCCESS;
 }
 else
  return EXIT_FAILURE;
}

template <int k,class T> BOOL Lktree<k,T>::isleaf(){
 int ct;
 BOOL r=TRUE;
 for(ct=0;(ct<k)&&r;ct++)
  if(sons[ct])
   r=FALSE;
 return r;
}

template <int k,class T> Lktree<k,T>* Lktree<k,T>::accessparent(){
 return parent;
}

template <int k,class T> Lktree<k,T>* Lktree<k,T>::accessson(int i){
 if((i<0)||(i>=k))
  return 0;
 else
  return sons[i];
}

template <int k,class T> unsigned Lktree<k,T>::getdepth(){
 return depth;
}

template <int k,class T> ostream& operator<<(ostream& s,Lktree<k,T> &T){
 int ct,ct1;
 if(T.depth>0)
  for(ct=0;ct<(T.depth-1)*2;ct++)
   s<<" ";
 if(T.depth>0)
  s<<"  ";
 s<<T.c<<endl;
 if(!T.isleaf())
  for(ct=0;ct<k;ct++)
   if(T.sons[ct])
    s<<*T.sons[ct];
   else{
    for(ct1=0;ct1<(T.depth)*2;ct1++)
     s<<" ";
    s<<"  (null)"<<endl;
   }
 return s;
}

/*
Radice
|--Zero
|  |--ZeroZero
|  `--(null)
`--Uno
   |--UnoZero
   |  |--UnoZeroZero
   |  `--(null)
   `--UnoUno

Radice
  Zero
    ZeroZero
    (null)
  Uno
    UnoZero
      UnoZeroZero
      (null)
    UnoUno
*/
#endif






