//This file is part of the Lstring v1.0 library written in -*- C++ -*-

//Lstring v1.0: a simple string class
//Copyright (C) Luca Saiu, 1999-2000

//This library is free software; you can redistribute it and/or modify it
//under the terms of the GNU Library General Public License version 2
//as published by the Free Software Foundation.

//This library is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Library General Public License for more details.

//You should have received a copy of the GNU Library General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


//Luca Saiu
//------------------------------------------
//e-mail:      luca78@dada.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "Lstring.h"
#include "../Lgen/Lgen.h"

extern "C"{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
}

Lstring::Lstring(){//To do: verify this is correct
 c=0;
 dim=0;
 zero='\0';
 operator=("");    //To do: this is a rough hack; verify first part is correct
}

Lstring::Lstring(const char x){
 zero='\0';        //Taken from default constructor
 c=(char*)xmalloc(20*sizeof(char)); //20 char's should be enough...
 sprintf(c,"%c",x);
 dim=strlen(c)+1;
}

Lstring::Lstring(const int x){
 zero='\0';        //Taken from default constructor
 c=(char*)xmalloc(20*sizeof(char)); //20 char's should be enough...
 sprintf(c,"%i",x);
 dim=strlen(c)+1;
}

Lstring::Lstring(const bool x){
 zero='\0';        //Taken from default constructor
 c=(char*)xmalloc(6*sizeof(char)); //6 char's are enough...
 if(x)
   sprintf(c,"TRUE",x);
 else
   sprintf(c,"FALSE",x);
 dim=strlen(c)+1;
}

Lstring::Lstring(const long x){
 zero='\0';        //Taken from default constructor
 c=(char*)xmalloc(20*sizeof(char)); //20 char's should be enough...
 sprintf(c,"%i",x);
 dim=strlen(c)+1;
}

Lstring::Lstring(const float x){
 zero='\0';        //Taken from default constructor
 c=(char*)xmalloc(20*sizeof(char)); //20 char's should be enough...
 sprintf(c,"%f",x);
 dim=strlen(c)+1;
}

Lstring::Lstring(const double x){
 zero='\0';        //Taken from default constructor
 c=(char*)xmalloc(20*sizeof(char)); //20 char's should be enough...
 sprintf(c,"%f",x);
 dim=strlen(c)+1;
}

Lstring::Lstring(const Lstring &s){
 dim=s.dim;
 c=(char*)xmalloc(dim*sizeof(char));
 memcpy(c,s.c,dim*sizeof(char));
 zero='\0';
}

Lstring::Lstring(const char *s){
  if(s==0)
    s="";
  c=(char*)xmalloc((dim=strlen(s)+1)*sizeof(char));
  memcpy(c,s,dim*sizeof(char));
  zero='\0';
}

Lstring& Lstring::operator=(Lstring s){
 if(c)
  free(c);
 c=(char*)xmalloc((dim=s.dim)*sizeof(char));
 memcpy(c,s.c,dim*sizeof(char));
 return *this;
}

Lstring& Lstring::operator=(char *s){
 if(c)
  free(c);
 c=(char*)xmalloc(sizeof(char)*(dim=(strlen(s)+1)));
 memcpy(c,s,dim*sizeof(char));
 return *this;
}

Lstring::~Lstring(){
 free(c);
}

const char* Lstring::ToCharStar() const{
 char *r=c;
 return r;
}

char& Lstring::operator[](const INDEX x){
 if(x<dim)
  return c[x];
 else return zero;
}

const char& Lstring::operator[](const INDEX x) const{
 if(x<dim)
  return c[x];
 else return zero;
}

Lstring Lstring::operator+=(Lstring s){
 if(s.dim){
  c=(char*)xrealloc(c,sizeof(char)*(dim+s.dim-1));
  memcpy(&(c[dim-1]),s.c,sizeof(char)*s.dim);
  dim+=s.dim-1;
 }
 return *this;
}

Lstring Lstring::operator+=(char c){
 char s[2];
 s[0]=c;
 s[1]='\0';
 return operator+=(s);
}

Lstring Lstring::operator--(){
 //To do: verify it's ok
 if(dim==1)
  return *this;
 dim--;
 c[dim-1]='\0';
 c=(char*)xrealloc(c,sizeof(char)*(dim));
 return *this;
}

Lstring Lstring::operator--(int){
 return operator--();
}

const Lstring operator+(const Lstring s1,const Lstring s2){
 Lstring r;
 r.dim=s1.dim+s2.dim-1;
 r.c=(char*)xmalloc(sizeof(char)*r.dim);
 memcpy(r.c,s1.c,sizeof(char)*s1.dim);
 strcat(r.c,s2.c);
 return r;
}

bool Lstring::operator==(const Lstring s) const{
 return (!strcmp(c,s.c));
}

bool Lstring::operator!=(const Lstring s) const{
 return strcmp(c,s.c);
}

bool Lstring::operator<(const Lstring s) const{
 return (strcmp(c,s.c)<0);
}

bool Lstring::operator>(const Lstring s) const{
 return (strcmp(c,s.c)>0);
}

bool Lstring::operator<=(const Lstring s) const{
 return ((operator<(s))||(operator==(s)));
}

bool Lstring::operator>=(const Lstring s) const{
 return ((operator>(s))||(operator==(s)));
}

const int Lstring::getdim() const{
 return dim-1;
}

void Lstring::setdim(const int n){
  int i;

  if(n==getdim())
    return;
  if(n>getdim())
    for(i=getdim();i<n;i++)
      operator+=(' ');
  else{
    Lstring s="";
    for(i=0;i<n;i++)
      s+=c[i];
    operator=(s);
  }
  return;
}

ostream& operator<<(ostream &stre,Lstring s){
 stre<<s.c;
 return stre;
}

istream& operator>>(istream &stre,Lstring &s){
 char c;

 s="";
 while(1){
   stre>>c;
   if(!stre.eof())
     s+=c;
   else
     break;//exit from while(1)
 }
 return stre;
}

const Lstring Lstring::ToUpper() const{
  Lstring r=*this;
  int i;
  for(i=0;i<r.getdim();i++)
    if(isalpha(r[i]))
      r[i]=toupper(r[i]);
  return r;
}

const Lstring Lstring::ToLower() const{
  Lstring r=*this;
  int i;
  for(i=0;i<r.getdim();i++)
    if(isalpha(r[i]))
      r[i]=tolower(r[i]);
  return r;
}

const bool Lstring::Tobool() const{
  Lstring up=ToUpper();
  if((up==Lstring("YES"))||(up==Lstring("TRUE"))||(up==Lstring("1"))||
     (up==Lstring("ON"))||(up==Lstring("VERO"))||(up==Lstring("V"))||
     (up==Lstring("Y"))||(up==Lstring("T"))||(up==Lstring("SI"))||
     (up==Lstring("SI\'"))||(up==Lstring("S")))
    return true;
  else
    return false;
}

const long Lstring::ToLong() const{
  return strtol(c,0,10);
}

const float Lstring::ToFloat() const{
  float r;
  sscanf(c,"%f",&r);
  return r;
}

const LArray<Lstring> Lstring::Tokenize(char separator) const{
  LArray<Lstring> r;
  int charcounter,indexcounter=0;

  // Initialize the first element
  r[0]="";
  for(charcounter=0;charcounter<getdim();charcounter++)
    if(c[charcounter]!=separator)      // if this char is not whitespace...
      r[indexcounter]+=c[charcounter]; // ...then add it to the token...
    else
      r[(++indexcounter)]="";          // ...else start a new token
  return r;
}

const Lstring Lstring::Invert() const{
  Lstring r=(*this);
  int i;

  for(i=0;i<r.getdim();i++)
    r[i]=c[r.getdim()-1-i];
  r[r.getdim()]='\0';
  return r;
}

const Lstring Lstring::NoSpacesBefore() const{
  Lstring r="";
  int i=0;

  while(((c[i]==' ')||(c[i]=='\t')) && (i<getdim()))
    i++;
  while(i<getdim()){
    r+=c[i];
    i++;
  }
  return r;
}

const Lstring Lstring::NoSpacesAfter() const{
  Lstring r=*this;

  while((r[r.getdim()-1]==' ')||(r[r.getdim()-1]=='\t'))
    r--;
  return r;
}

const Lstring Lstring::NoSpacesAround() const{
  return NoSpacesBefore().NoSpacesAfter();
}

const Lstring Lstring::WordWrap(int n) const{
  // If the string is less than a line long then there's nothing to do, and
  // n<=0 is an error:
  if((getdim()<=n)||(n<=0))
    return *this;

  // If the string already contains a newline character, then recursively
  // apply the method to both segments:

  int i; // loop counter
  for(i=0;i<getdim();i++)
    if(c[i]=='\n')
      return FromTo(0,i-1).WordWrap(n)+
             NEWLINE+
             FromTo(i+1,getdim()-1).WordWrap(n);

  // If I'm here I'm sure there are no '\n' in the string.

  Lstring r;
  Lstring temp=*this;

  do{
    for(i=n;(temp[i]!=' ')&&(i>=0);i--)
      ; // do nothing

    if(temp[i]!=' '){
      // This is a very long word, longer than n: we must break it:
      i=n-1;
      // One of these characters will be erased:
      temp=temp.FromTo(0,i-2)+"--"+temp.FromTo(i-1,temp.getdim()-1);
    }
    
    // Copy from temp[0] to temp[n-1]:
    if(r=="")
      r=temp.FromTo(0,i-1);
    else
      r+=NEWLINE+temp.FromTo(0,i-1);

    // Now take a new segment of the initial string:
    temp=temp.FromTo(i+1,temp.getdim()-1);
  }while(temp.getdim()>n);

  return r+NEWLINE+temp;
}

const Lstring Lstring::FromTo(int a,int b) const{
  // Test for error conditions:
  if((a>=b)||(b>getdim())||(a<0))
    return "";

  // Ok, a<b<=dimesion; now write the segment into r...
  int i;
  Lstring r;
  for(i=a;i<=b;i++)
    r+=c[i];

  // ...and return it:
  return r;
}

int Lstring::Lines() const{
  int i;  // loop counter
  int r=1;

  // Count the '\n' characters and write the value in r:
  for(i=0;i<getdim();i++)
    if(c[i]=='\n')
      r++;

  // Return r:
  return r;
}

int Lstring::LongestLineLength() const{
  int i,j;  // loop counters
  int thislinelength=0,max=0;

  // Scan every character:
  for(i=0;i<getdim();i++){
    if(c[i]!='\n')      // If this is a normal character...
      thislinelength++; // ...increment thislinelength.
    else{
      // A line is finished: update max if needed:
      if(thislinelength>max)
        max=thislinelength;

      // Zero thislinelength, for the next iteration:
      thislinelength=0;
    }
  }

  // Update max for the last line, if needed:
  if(thislinelength>max)
    max=thislinelength;

  return max;
}

bool Lstring::Contains(char ch) const{
  // Scan c and return true if you find ch:
  int i;
  for(i=0;i<getdim();i++)
    if(c[i]==ch)
      return true;

  // If we are here then we have not found ch:
  return false;
}

int Lstring::FindSubstring(const Lstring s) const{
  int i,j;
  for(i=0;i<getdim();i++){
    if(c[i]==s[0]){ // If the first character matches...
      // ...Then look at the other ones:
      for(j=0;j<s.getdim();j++)
        if(c[i+j]!=s[j]) // If a character after the first doesn't match...
          goto label;    // try another i:

      // If we are here then the whole s matches:
      return i;
    }
    label: ; // label must be followed by a statement, and ';' is a statement
  }

  // We didn't find the substring:
  return -1;
}

const Lstring Lstring::Replace(const Lstring a,const Lstring b){

  int i=FindSubstring(a);
  if(i==-1)  // If a is not a substring of *this then do nothing:
    return *this;
  else       // A simple recursive call:
    *this=FromTo(0,i-1)+b+
          (Lstring(FromTo(i+a.getdim(),getdim()-1)
                  ).Replace(a,b));
  return *this;
}

const Lstring Lstring::ReplaceInPlace(const Lstring a,const Lstring b) const{
  Lstring r=*this;
  int i=r.FindSubstring(a);
  if(i==-1)  // If a is not a substring of *this then do nothing:
    return r;
  else       // A simple recursive call:
    r=r.FromTo(0,i-1)+b+
      (Lstring(r.FromTo(i+a.getdim(),r.getdim()-1)
              ).Replace(a,b));
  return r;
}

const Lstring Lstring::FromLineToLine(int a,int b) const{
  int i,j;      // loop counters
  int lines=0;  // line counter

  // Locate the a-th line:
  for(i=0;(i<getdim())&&(lines<a);i++)
    if(c[i]=='\n')
      lines++;
      
  // The next character is the first of the a-th line.

  // Now compose the string to return, and stop at the b-th line or at the
  // end of the string:
  Lstring r;
  for(j=i;(j<getdim())&&(lines<=b);j++){
    r+=c[j];
    if(c[j]=='\n')
      lines++;
  }

  // Return the composed string:
  return r;
}

